/**********************************************************************
*  ______   ________   ___   __    ________   ___   ___   ______      *
* /_____/\ /_______/\ /__/\ /__/\ /_______/\ /___/\/__/\ /_____/\     *      
* \:::_ \ \\::: _  \ \\::\_\\  \ \\::: _  \ \\::.\ \\ \ \\:::_ \ \    *   
*  \:(_) \ \\::(_)  \ \\:. `-\  \ \\::(_)  \ \\:: \/_) \ \\:\ \ \ \   * 
*   \: ___\/ \:: __  \ \\:. _    \ \\:: __  \ \\:. __  ( ( \:\ \ \ \  * 
*    \ \ \    \:.\ \  \ \\. \`-\  \ \\:.\ \  \ \\: \ )  \ \ \:\_\ \ \ * 
*     \_\/     \__\/\__\/ \__\/ \__\/ \__\/\__\/ \__\/\__\/  \_____\/ *
*                                                                     *
***********************************************************************
*                                                                     *
*                              Panako                                 * 
*                       Acoustic Fingerprinting                       *
*                                                                     *
***********************************************************************/
package be.panako.cli;

import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.UnsupportedAudioFileException;

import be.hogent.tarsos.dsp.AudioDispatcher;
import be.hogent.tarsos.dsp.AudioFile;
import be.hogent.tarsos.dsp.AudioPlayer;
import be.hogent.tarsos.dsp.util.AudioResourceUtils;


public class Play extends Application {
	
	
	private static final int TARGET_SAMPLE_RATE = 15649;

	@Override
	public void run(String... args) {
		String inputResource = AudioResourceUtils.sanitizeResource(args[0]);
		AudioDispatcher d;
		try {
			AudioFile audioFile = new AudioFile(inputResource);
			AudioInputStream stream = audioFile.getMonoStream(TARGET_SAMPLE_RATE);
			d = new AudioDispatcher(stream,2028,0);
			d.addAudioProcessor(new AudioPlayer(stream.getFormat()));
			d.run();
		} catch (UnsupportedAudioFileException e) {
			e.printStackTrace();
			System.err.print(e.getLocalizedMessage());
		}  catch (LineUnavailableException e) {
			e.printStackTrace();
			System.err.print(e.getLocalizedMessage());
		}
	}

	@Override
	public String description() {
		return "Plays an audio file or stream. It is used to test the audio conversion system. "
				+ " \n If the JVM and decoder are configured well, it should play (http) streams and audio from container formats."
				+ "";
	}

	@Override
	public String synopsis() {
		return "play [xspf|pls|m3u|asx|avi|file|stream]";
	}
	
	@Override
	public boolean needsStorage() {
		return false;
	}

	@Override
	public boolean writesToStorage() {
		return false;
	}
}
