package be.panako.strategy;

public class QueryResult {
	
	public final String identifier;
	public final String description;
	public final double score;
	public final double time;
	public final double timeFactor;
	public final double frequencyFactor;
	
	/**
	 * @param identifier
	 *            The internal identifier of the matched audio
	 * @param description
	 *            The meta-data, description of the matched audio
	 * @param score
	 *            The score for the match
	 * @param time
	 *            The starting position in the matched audio, in seconds.
	 **/            
	public QueryResult(String identifier,String description,double score,double time){
		this(identifier,description,score,time,1.0,1.0);
		
	}
	
	/**
	 * @param identifier
	 *            The internal identifier of the matched audio
	 * @param description
	 *            The meta-data, description of the matched audio
	 * @param score
	 *            The score for the match
	 * @param time
	 *            The starting position in the matched audio, in seconds.
	 * @param timeFactor
	 *            The factor (percentage) of change in time. 110 means 10%
	 *            speedup compared to the reference. 90 means 10% slower than
	 *            reference.
	 * @param frequencyFactor
	 *            The factor (percentage) of change in frequency. 110 means 10%
	 *            higher frequency compared to the reference. 90 means a 10%
	 *            lower frequency.
	 */
	public QueryResult(String identifier, String description, double score, double time,double timeFactor, double frequencyFactor){
		this.identifier = identifier;
		this.description = description;
		this.score = score;
		this.time = time;
		this.timeFactor=timeFactor;
		this.frequencyFactor = frequencyFactor;
	}
	
}
