/**********************************************************************
*  ______   ________   ___   __    ________   ___   ___   ______      *
* /_____/\ /_______/\ /__/\ /__/\ /_______/\ /___/\/__/\ /_____/\     *      
* \:::_ \ \\::: _  \ \\::\_\\  \ \\::: _  \ \\::.\ \\ \ \\:::_ \ \    *   
*  \:(_) \ \\::(_)  \ \\:. `-\  \ \\::(_)  \ \\:: \/_) \ \\:\ \ \ \   * 
*   \: ___\/ \:: __  \ \\:. _    \ \\:: __  \ \\:. __  ( ( \:\ \ \ \  * 
*    \ \ \    \:.\ \  \ \\. \`-\  \ \\:.\ \  \ \\: \ )  \ \ \:\_\ \ \ * 
*     \_\/     \__\/\__\/ \__\/ \__\/ \__\/\__\/ \__\/\__\/  \_____\/ *
*                                                                     *
***********************************************************************
*                                                                     *
*                              Panako                                 * 
*                       Acoustic Fingerprinting                       *
*                                                                     *
***********************************************************************/

package be.panako.strategy.fft.storage;


/**
 * Describes a landmark hit in the store.
 * @author Joren Six
 */
public class FFTFingerprintHit {
	
	/**
	 * The audio identifier
	 */
	public int identifier;
	
	/**
	 * Time in blocks in the original, matched audio.
	 */
	public int matchTime;
	
	/**
	 * Difference in time between the original, matched audio and the query. Expressed in blocks.
	 */
	public int timeDifference;	
	
	/**
	 * Time in blocks in the query.
	 */
	public int queryTime;
	
	
	
	public boolean equals(Object other){
	    if (other == null){
	    	return false;
	    }
	    if (other == this){
	    	return true;
	    }
	    if (!(other instanceof FFTFingerprintHit)){
	    	return false;
	    }
	    FFTFingerprintHit otherFingerprintHit = (FFTFingerprintHit) other;
	    return otherFingerprintHit.hashCode() == this.hashCode();
	}
	
	
	/*
	 * It is the same fingerprint hit if query time, match time and identifier are the same.
	 */
	public int hashCode(){
		return matchTime | identifier | queryTime;
	}
	
}
